#include <stdlib.h>
#include <android/bitmap.h>

#include "ComicFinder.hpp"

JNIEXPORT bool JNICALL Java_com_example_comicfinder_MainActivity_processImage
    (JNIEnv *env, jobject me, jbyteArray src, jobject dst, jint width, jint height)
{
    int ret;
    unsigned char *p_dst;
    if ((ret = AndroidBitmap_lockPixels(env, dst, reinterpret_cast<void **>(&p_dst))) < 0) {
        return false;
    }

    unsigned char *p_src = reinterpret_cast<unsigned char *>(env->GetPrimitiveArrayCritical(src, NULL));
    if (NULL == p_src) {
        AndroidBitmap_unlockPixels(env, dst);
        return false;
    }

    const int threshold_black_and_gray = 60;
    const int threshold_gray_and_white = 120;
    const int threshold_edge = 32;

    int lum;
    for (int i = 0; i < height; i++) {
        for (int j = 0; j < width; j++) {
            lum = p_src[i*width+j];

            if (lum < threshold_black_and_gray) {
                lum = 0x00;
            }
            else if (lum < threshold_gray_and_white) {
                lum = ((i+j) % 2) == 0 ? 0x00 : 0xff;
            }
            else {
                lum = 0xff;
            }

            p_dst[(i*width+j)*4+0] = p_dst[(i*width+j)*4+1] = p_dst[(i*width+j)*4+2] = lum;
            p_dst[(i*width+j)*4+3] = 0xff;
        }
    }

    int fx, fy, im, ip, jm, jp;
    for (int i = 0; i < height; i++) {
        im = i-1;
        ip = i+1;
        im = im < 0 ? 0 : im;
        ip = ip > height-1 ? height-1 : ip;

        for (int j = 0; j < width; j++) {
            jm = j-1;
            jp = j+1;
            jm = jm < 0 ? 0 : jm;
            jp = jp > width-1 ? width-1 : jp;

            fx = 0;
            fx += p_src[im*width+jm] * -1;
            fx += p_src[i *width+jm] * -2;
            fx += p_src[ip*width+jm] * -1;
            fx += p_src[im*width+jp] *  1;
            fx += p_src[i *width+jp] *  2;
            fx += p_src[ip*width+jp] *  1;
            fy = 0;
            fy += p_src[im*width+jm] * -1;
            fy += p_src[im*width+j ] * -2;
            fy += p_src[im*width+jp] * -1;
            fy += p_src[ip*width+jm] *  1;
            fy += p_src[ip*width+j ] *  2;
            fy += p_src[ip*width+jp] *  1;
            lum = abs(fx) + abs(fy);
            lum = lum < threshold_edge ? 0xff : 0x00;

            p_dst[(i*width+j)*4+0] = p_dst[(i*width+j)*4+1] = p_dst[(i*width+j)*4+2] = lum & p_dst[(i*width+j)*4+0];
            p_dst[(i*width+j)*4+3] = 0xff;
        }
    }

    env->ReleasePrimitiveArrayCritical(src, p_src, 0);
    AndroidBitmap_unlockPixels(env, dst);

    return true;
}
